/*! @file echo_client.c
 *  @brief Simples aplicação em linguagem C.
 *  @details Implementa um cliente que envia uma sequência de caracter.
 *  @author Norberto R. de Goes Jr.
 *  @bug nenhum bug conhecido.
 */

#include "utils.h"
#include <time.h>

/** Estrutura de endereço para o servidor. */
struct sockaddr_in server_g;

/**
 * Gerador de sequência de caracteres (string) de forma aleatória .
 * @param p_string Ponteiro para o buffer que receberá a string.
 * @param max_size Tamanho máximo da string.
 * @return A quantidade de caracteres na sequência gerada.
 */
unsigned int generate_random(char* p_string, unsigned int max_size) {

    static const char alphanum[] =
            "0123456789"
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    int i = 0;
    int random; // = rand();

    random = max_size; //random % max_size;

    for (; i < random; ++i) {
        p_string[i] = alphanum[rand() % (sizeof(alphanum) - 1)];
    }
    p_string[random] = 0;
    return random+1;
}

/**
 * Conexão com o servidor.
 */
void connect_server(int socket_fd) {

    while (connect(socket_fd, (struct sockaddr *)&server_g,
            sizeof(server_g)) != 0) {
        perror("Falha na conexao. Retentando...");
        sleep (2);
    }
}

/**
 * Abertura do socket.
 * @param p_ip_address Ponteiro para o endereço IP do servidor.
 * @param port Porta do servidor.
 */
int open_connect_socket() {

    int socket_fd;

    //Abre o socket servidor:
    open_socket(&socket_fd, SOCK_STREAM, 0 /* timeout*/);

    printf("Conectando ao servidor %s...\n", ip_address_server_g);

    server_g.sin_addr.s_addr = inet_addr(ip_address_server_g);
    server_g.sin_family = AF_INET;
    server_g.sin_port = htons(server_port_tcp_g);
    // conecta ao servidor:
    connect_server(socket_fd);

    printf("Conexão OK  ao servidor\n\n\n");

    return socket_fd;
}

/**
 * Implementação do cliente propriamente dito (echo-loop).
 */
void* echo_client(void * p_param) {

    char message_out[MAX_DATA_SIZE];

    int client_socket = -1;
    char init = 1;
    int length;
    unsigned int counter_tx = 0;

    srand(time(NULL)); // p/ geração aleatoria de strings.

    while(1)  { // "for ever"...
        if (init) {
            if (client_socket != -1) {
                close(client_socket);
            }
            client_socket = open_connect_socket();
            init = 0;
        }

        length = generate_random(message_out, data_size_tx_g);

        //envia mensagem:
        if (send(client_socket, message_out, length, 0) > 0) {
            printf ("Send OK - %d\n", ++counter_tx);
            // no wait reply
            usleep (200*1000);
        } else {
            perror("Erro envio");
            init = 1;
        }
    }
    return NULL;
}
