/**
 * Função main.
 * @param argc Número de argumentos de entrada (chamada da função).
 * @param argv Ponteiro para os valores dos argumentos.
 * @return Valor EXIT_SUCCESS se programa executado com sucesso.
 */
#include <pthread.h>
#include "utils.h"

int  echo_server_tcp_socket_g = -1;
char running_g = 1;

/**
 * Handler para saída (ctrl-C).
 * @param signo Identificação do sinal (kill).
 */
void sig_handler(int signo) {
    if (signo == SIGINT) {
        printf("\n\nExiting...\n\n");
        running_g = 0;
        close(echo_server_tcp_socket_g);
        sleep(1);
        exit(EXIT_SUCCESS); // return OK.
    }
}
/**
 * Instala o tratador do sinal "ctrl-c" (kill -2)
 */
void attach_signal(void) {
    if (signal(SIGINT, sig_handler) == SIG_ERR) {
        perror("Error - catch SIGINT");
        exit(EXIT_FAILURE);
    }
}


int main(int argc , char* argv[]) {

    pthread_t thread_id_client;

    attach_signal(); // instala tratador de ctrl-c .

    // "parser" dos parâmetros de entrada :
    get_args(argc, argv);

    // execução:
    printf("Iniciando cliente ECHO...\n");
    if (pthread_create(&thread_id_client, NULL, echo_client, NULL)) {
        fprintf(stderr, "Error creating client thread. Exiting...\n");
        exit(EXIT_SUCCESS);
    } else {
        printf("Cliente ECHO iniciado\n");
    }

    pthread_join(thread_id_client, NULL);

    exit(EXIT_SUCCESS);
}
