/*! @file utils.c
 *  @brief Contém o métodos comuns aos modulos.
 *  @details
 *  @author Norberto R. de Goes Jr.
 *  @bug nenhum bug conhecido.
 */

#include "utils.h"
#include <libgen.h>

char ip_address_server_g[16];
int  server_port_tcp_g = -1;
int  data_size_tx_g = 512;

void print_usage(char *app_name) {
    printf("\nUsage:\n"
            "   %s server_ip_address server_port data_size\n"
            "      Exemplos: \n"
            "             %s 10.202.46.5 5124 256   |   Exiting...\n\n\n",
            basename(app_name), basename(app_name));
    exit(EXIT_FAILURE);
}

char has_args_parameter(const char *parameter, int argc, char **argv) {

    int i;
    for (i = 1;  i < argc; i++) {
        if (!strcmp(parameter, argv[i])) {
            return 1;
        }
    }
    return 0; // parameter not found.
}

void get_args(int argc, char* argv[]) {

    if (argc != 4) { // erro e saida:
        print_usage(argv[0]);
    }
    // especificos:
    strcpy (ip_address_server_g, argv[1]);
    server_port_tcp_g = atoi(argv[2]);
    data_size_tx_g = atoi(argv[3]);
    return;

    print_usage(argv[0]); // erro e saida...
}

/**
 * Abertura do socket.
 * @param p_socket_fd Ponteiro para o valor do socket em tratamento.
 * @param type Tipo do socket, que especifica a semântica de comunicação. Exemplos: SOCK_STREAM, SOCK_DGRAM.
 * @param use_timeout Quando diferente de zero, indica que é para setar o "timeout" de recepçao.
 */
void open_socket(int* p_socket_fd, int type, char use_timeout) {

    struct timeval timeout;

    if (*p_socket_fd != -1) {
        close(*p_socket_fd);
    }
    //Create socket
    *p_socket_fd = socket(AF_INET, type , 0);

    if (*p_socket_fd == -1)  {
        perror("Erro na criacao do socket");
        exit(EXIT_FAILURE);
    }

    if (use_timeout) {
        timeout.tv_sec  = 2;
        timeout.tv_usec = 0;

        if (setsockopt (*p_socket_fd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
                sizeof(timeout)) < 0) {
            puts("Setsockopt falha - rcv");
            exit(EXIT_FAILURE);
        }
    }
    printf("\n\nCriação de socket OK\n");
}
/**
 * Incrementa e verifica se o valor do contador atingiu valor definido.
 * @param p_counter Ponteiro para o contador.
 * @return Valor 1 se valor atingido, caso contrario, 0.
 */
char inc_check_counter(unsigned int* p_counter, unsigned int limit) {

    if (*p_counter == 0) {
        *p_counter = limit;
    } else {
        (*p_counter)--;
    }
    return *p_counter == limit;
}
